#ifndef SELFPATH_C
#define SELFPATH_C
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#ifdef __TEST_ST_SELF
#include <stdio.h> /* for testing only */
#include <stdlib.h> /* for testing only */
#endif /* __TEST_ST_SELF */
#include "bname.c"
#include "envget.c"
#include "psearch.c"
/*
  Typically $_ or argv[0] are set to the names of the program the SHELL found
  to execute, either full path names or basenames.  When this data contains the
  full path we let it stay, leaving it to callers to find the suffix they want
  since they may be able to avoid path_search().
*/
static inline int selfpath(char *av0, char *soft, struct stat *sth, struct stat *sts) {
    char *p, path[4096];
    int npath;

    /* sh/ksh/zsh family sets $_ to full path of soft link. Trust this first. */
    if ((p=envget("_", (char*)0)) && p[0]=='/' && !strcmp(bname(p), bname(av0)))
	goto full_path_ready;
    /* argv[0] full path or explicit relative path -> just use that basename */
    if (av0[0] == '/') {
	p = av0;
	goto full_path_ready;
    }
    if (strncmp(av0, "./", 2) == 0 || strncmp(av0, "../", 3) == 0) {
#ifdef DISTRUST_PWD
	if (!getcwd(path, sizeof path))
	    return -1;
#else  /* DISTRUST_PWD */
	strcpy(path, envget("PWD", "./"));
#endif /* DISTRUST_PWD */
	npath = strlen(path);
	if (npath == sizeof path - 1)
	    return -1;
	path[npath] = '/';
	strncpy(path + npath + 1, av0, sizeof path - npath - 1);
	p = path;
	goto full_path_ready;
    }
    /* Need to do a path search for av0 */
    strcpy(soft, av0);
    if (path_search(soft, (struct stat *)0, sts) >= 0) {
	stat(soft, sth);
	return 0;
    }
    return -1;
full_path_ready:
    strcpy(soft, p);
    stat(soft, sth);
    lstat(soft, sts);
    return 0;
}

#ifdef __TEST_ST_SELF
int main(int ac, char **av) {
    char        softname[4096];
    struct stat st_soft,
                st_hard;

    if (selfpath(av[0], softname, &st_hard, &st_soft) >= 0)
	fprintf(stderr,
		"Names I think I am:\n"
		"  argv[0]=\"%s\"\n"
		"  hard   = (%d:%d)\n"
		"  soft   =\"%s\" (%d:%d)\n",
		av[0], (int)st_hard.st_dev, (int)st_hard.st_ino,
		softname, (int)st_soft.st_dev, (int)st_soft.st_ino);
    else
	return 1;
    return 0;
}
#endif /* __TEST_ST_SELF */
#endif /* SELF_PATH */
